@@ -1,6 +1,6 @@
 /*
 	conffile.cc	Configuration File Functions
-	Copyright (c) 2000, 2001, 2002, 2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2002,2003,2004,2005,2007 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -135,6 +135,11 @@ void	ExtraChar(const char *cmd, const char *buf)
 	PrintErrorMessage(_("extra characters"), cmd, buf);
 }
 
+void	CommaExpected(const char *cmd, const char *buf)
+{
+	PrintErrorMessage(_("comma expected"), cmd, buf);
+}
+
 void	InvalidValue(const char *cmd, const char *buf)
 {
 	PrintErrorMessage(_("invalid value"), cmd, buf);
@@ -150,6 +155,21 @@ void	InvalidColor(const char *cmd, const char *buf)
 	PrintErrorMessage(_("invalid color"), cmd, buf);
 }
 
+void	InvalidOption(const char *cmd, const char *buf)
+{
+	PrintErrorMessage(_("invalid option"), cmd, buf);
+}
+
+void	InvalidRegexOption(const char *cmd, const char *buf)
+{
+	PrintErrorMessage(_("invalid regex option"), cmd, buf);
+}
+
+void	NoRegex(const char *cmd, const char *buf)
+{
+	PrintErrorMessage(_("regular expression not supported"), cmd, buf);
+}
+
 void	MissingOn(const char *cmd, const char *buf)
 {
 	PrintErrorMessage(_("ON required"), cmd, buf);
@@ -264,8 +284,36 @@ class	IsPatternMatch {
 					// fnmatch returns 0 if strings match
 			return ! k_fnmatch(pattern, str->dir);
 		}
+		int	operator ()(MountDirItem &str)
+		{
+					// fnmatch returns 0 if strings match
+			return ! k_fnmatch(pattern, str->dir);
+		}
 };
 
+#ifdef	CLIB_HAVE_REGEX
+class	IsRegPatternMatch {
+		string	pattern;
+		regex_t	regex_buffer;
+	public:
+		IsRegPatternMatch(const string &pattern_) : pattern(pattern_) {
+			k_regcomp(&regex_buffer, pattern, REG_EXTENDED);
+		}
+		int	operator ()(DirItem &str)
+		{
+			regmatch_t m;
+					// regexec returns 0 if strings match
+			return !regexec(&regex_buffer, str->dir.c_str(), 1, &m, 0);
+		}
+		int	operator ()(MountDirItem &str)
+		{
+			regmatch_t m;
+					// regexec returns 0 if strings match
+			return !regexec(&regex_buffer, str->dir.c_str(), 1, &m, 0);
+		}
+};
+#endif
+
 /*************************************************************************
 	Command processing/printing value structure.
 *************************************************************************/
@@ -275,8 +323,10 @@ struct	ConfigValue {
 	COLOR_TYPE KcdConfig::*cfgColorPtr;
 	bool	KcdConfig::*cfgBoolPtr;
 	DirList	* KcdConfig::*cfgStringListPtr;
+	MountDirList *KcdConfig::*cfgMountDirListPtr;
 	AttrConfig KcdConfig::*cfgAttrPtr;
 	KeyBindingType KcdConfig::*cfgKeyPtr;
+	ShowNewDirType KcdConfig::*cfgShowNewDirPtr;
 };
 
 /*************************************************************************
@@ -384,7 +434,7 @@ void	PrintUnsigned(const char *cmd, const ConfigValue &cfg)
 *************************************************************************/
 
 struct	KeyBindingMap {
-	char		*name;
+	const char	*name;
 	KeyBindingType	value;
 };
 
@@ -450,16 +500,86 @@ void	PrintKeyBinding(const char *cmd, const ConfigValue &cfg)
 	cout << '\n';
 }
 
+/*************************************************************************
+	Command processing/printing functions - Show New Directory.
+*************************************************************************/
+
+struct	ShowNewDirMap {
+	const char	*name;
+	ShowNewDirType	value;
+};
+
+#define NUM_SHOW_NEW_DIR	4
+
+ShowNewDirMap	showNewDirMap[NUM_SHOW_NEW_DIR] = {
+	{ "default", showNewDirMulti },
+	{ "no", showNewDirNo },
+	{ "yes", showNewDirYes },
+	{ "multi", showNewDirMulti }
+};
+
+
+void	SetShowNewDir(const ConfigValue &cfg, ShowNewDirType k)
+{
+	allConfig[configProfile].*cfg.cfgShowNewDirPtr = k;
+}
+
+void	ConfigShowNewDir(const char *cmd, char *buf, size_t from, size_t len,
+		  const ConfigValue &cfg)
+{
+	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
+
+	size_t	i;
+	size_t	tok_end = SeekConfigTokenEnd(buf, from, len);
+
+	for (i = 0; i < NUM_SHOW_NEW_DIR; ++i) {
+		size_t key_len = strlen(showNewDirMap[i].name);
+		if (tok_end-from == key_len 
+		    && CompareStringCase(showNewDirMap[i].name, buf+from, key_len) == 0)
+			break;
+	}
+	if (i == NUM_SHOW_NEW_DIR)
+			// Throw exception
+		PrintErrorMessage(_("invalid attribute"), cmd, buf);
+
+	from = tok_end;
+	from = SeekNonSpace(buf, from, len);
+	if (from != len)
+		ExtraChar(cmd, buf);
+
+	ShowNewDirType k = showNewDirMap[i].value;
+		
+			// Note: error has already been reported
+
+	SetShowNewDir(cfg, k);
+}
+
+void	PrintShowNewDir(const char *cmd, const ConfigValue &cfg)
+{
+	cout << cmd << ": ";
+	switch (allConfig[configProfile].*cfg.cfgShowNewDirPtr) {
+		case showNewDirNo:
+			cout << "No";
+			break;
+		case showNewDirYes:
+			cout << "Yes";
+			break;
+		case showNewDirMulti:
+			cout << "Multi";
+			break;
+	}
+	cout << '\n';
+}
+
 /*************************************************************************
 	Command processing/printing functions - String list.
 *************************************************************************/
 
-void	SetStringList(const ConfigValue &cfg, const string &s)
+void	SetStringList(const ConfigValue &cfg, const string &s, bool regex,
+		      bool globDot, bool globPath)
 {
 						// Reference counted
-	DirItem	str(new DirConfig(s, 
-				  allConfig[configProfile].cfgGlobDot, 
-				  allConfig[configProfile].cfgGlobPath));
+	DirItem	str(new DirConfig(s, regex, globDot, globPath));
 
 	DirList::iterator iter;
 	for (iter = (allConfig[configProfile].*cfg.cfgStringListPtr)->begin(); 
@@ -488,6 +608,16 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 {
 	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
 
+	bool regex = false;
+	if (buf[from] == 'r' || buf[from] == 'R') {
+#ifdef	CLIB_HAVE_REGEX
+		regex = true;
+		from = SeekNonSpace(buf, from+1, len);
+#else
+		NoRegex(cmd, buf);
+#endif
+	}
+
 	if (buf[from] != '\"')
 		QuoteNotFound(cmd, buf);
 	from++;
@@ -495,9 +625,6 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 					// Shell-style quote seek
 	size_t end_quote = SeekEndQuoteOrThrow(cmd, buf, from, len);
 
-	if (!IsEmpty(buf, end_quote+1, len))
-		ExtraChar(cmd, buf);
-
 					// Command argument is "" ?	
 	if (from == end_quote)
 		BlankEntry(cmd, buf);
@@ -508,21 +635,100 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 		PrintErrorMessage(_("cannot skip root directory"), cmd, buf);
 	}
 
-					// Now that command is valid, we can
-					// modify buf and process the command
-
-					// Remove the closing quote
-	buf[end_quote] = 0;
-					
+					// Process directory
+	string	dir(buf+from, end_quote-from);
 	string	expanded_dir;
 	try {
-		expanded_dir = ExpandDirectory(buf+from, true);
+		expanded_dir = ExpandDirectory(dir, true, regex);
+
+		if (cfg.cfgStringListPtr == &KcdConfig::cfgSkipDir)
+			expanded_dir = ResolveSymlink(expanded_dir);
 	}
 	catch (ErrorGenericSyntax &e) {
 		PrintErrorMessage(e.what(), cmd, buf);
 	}
 
-	SetStringList(cfg, expanded_dir);
+	bool globDot = allConfig[configProfile].cfgGlobDot;
+	bool globPath = allConfig[configProfile].cfgGlobPath;
+
+	from = SeekNonSpace(buf, end_quote+1, len);
+	while (!IsEmpty(buf, from, len)) {
+		if (buf[from] != ',')
+			CommaExpected(cmd, buf);
+
+		from = SeekNonSpace(buf, from+1, len);
+		size_t tok_end = SeekConfigTokenEnd(buf, from, len);
+		bool *b = 0;
+		if (tok_end-from == 7
+		    && CompareStringCase("GlobDot", buf+from, 7) == 0) {
+			if (regex)
+				InvalidRegexOption(cmd, buf);
+			b = &globDot;
+		}
+		else if (tok_end-from == 8
+		    && CompareStringCase("GlobPath", buf+from, 8) == 0) {
+			if (regex)
+				InvalidRegexOption(cmd, buf);
+			b = &globPath;
+		}
+		else
+			InvalidOption(cmd, buf);
+
+		from = SeekPastEqualSignOrThrow(cmd, buf, tok_end, len);
+
+		if (CompareStringCase(buf+from, "yes", 3) == 0) {
+			*b = true;
+			from += 3;
+		}
+		else if (CompareStringCase(buf+from, "no", 2) == 0) {
+			*b = false;
+			from += 2;
+		}
+		else
+			YesNoNotFound(cmd, buf);
+	}
+
+	SetStringList(cfg, expanded_dir, regex, globDot, globPath);
+}
+
+void	ConfigStringListNoGlob(const char *cmd, char *buf, size_t from, size_t len, 
+			       const ConfigValue &cfg)
+{
+	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
+
+	if (buf[from] != '\"')
+		QuoteNotFound(cmd, buf);
+	from++;
+
+					// Shell-style quote seek
+	size_t end_quote = SeekEndQuoteOrThrow(cmd, buf, from, len);
+
+					// Command argument is "" ?	
+	if (from == end_quote)
+		BlankEntry(cmd, buf);
+
+					// Attempt to skip "/" ?
+	if (cfg.cfgStringListPtr == &KcdConfig::cfgSkipDir
+	    && strncmp("/", buf+from, end_quote-from) == 0) {
+		PrintErrorMessage(_("cannot skip root directory"), cmd, buf);
+	}
+
+	if (!IsEmpty(buf, end_quote+1, len))
+		ExtraChar(cmd, buf);
+
+					// Process directory
+	string	dir(buf+from, end_quote-from);
+	string	expanded_dir;
+	try {
+		expanded_dir = ExpandDirectory(dir, true);
+	}
+	catch (ErrorGenericSyntax &e) {
+		PrintErrorMessage(e.what(), cmd, buf);
+	}
+
+	SetStringList(cfg, expanded_dir, false,
+		      allConfig[configProfile].cfgGlobDot, 
+		      allConfig[configProfile].cfgGlobPath);
 }
 
 void	PrintStringList(const char *cmd, const ConfigValue &cfg)
@@ -541,7 +747,9 @@ void	PrintStringList(const char *cmd, const ConfigValue &cfg)
 		else
 			indent = true; 
 		cout << (*iter)->dir;
-		if ((*iter)->globDot || (*iter)->globPath) {
+		if ((*iter)->regex)
+			cout << "   [ Regex ]";
+		else if ((*iter)->globDot || (*iter)->globPath) {
 			cout << "   [";
 			if ((*iter)->globDot)
 				cout << " GlobDot";
@@ -589,9 +797,14 @@ void	SetClearStringList(const ConfigValue &cfg)
 	(allConfig[configProfile].*cfg.cfgStringListPtr)->clear();
 }
 
-void	SetClearStringList(const ConfigValue &cfg, const string &s)
+void	SetClearStringList(const ConfigValue &cfg, const string &s, bool regex)
 {
-	(allConfig[configProfile].*cfg.cfgStringListPtr)->remove_if(IsPatternMatch(s));
+#ifdef	CLIB_HAVE_REGEX
+	if (regex)
+		(allConfig[configProfile].*cfg.cfgStringListPtr)->remove_if(IsRegPatternMatch(s));
+	else
+#endif
+		(allConfig[configProfile].*cfg.cfgStringListPtr)->remove_if(IsPatternMatch(s));
 }
 
 void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len, 
@@ -603,6 +816,16 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 
 					// Expect directory name
 
+		bool regex = false;
+		if (buf[from] == 'r' || buf[from] == 'R') {
+#ifdef	CLIB_HAVE_REGEX
+			regex = true;
+			from = SeekNonSpace(buf, from+1, len);
+#else
+			NoRegex(cmd, buf);
+#endif
+		}
+
 		if (buf[from] != '\"')
 			QuoteNotFound(cmd, buf);
 		from++;
@@ -617,21 +840,17 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 		if (from == end_quote)
 			BlankEntry(cmd, buf);
 
-					// Now that command is valid, we can
-					// modify buf and process the command
-
-					// Remove the closing quote
-		buf[end_quote] = 0;
-
+					// Process directory
+		string	dir(buf+from, end_quote-from);
 		string	expanded_dir;
 		try {
-			expanded_dir = ExpandDirectory(buf+from, true);
+			expanded_dir = ExpandDirectory(dir, true, regex);
 		}
 		catch (ErrorGenericSyntax &e) {
 			PrintErrorMessage(e.what(), cmd, buf);
 		}
 
-		SetClearStringList(cfg, expanded_dir);
+		SetClearStringList(cfg, expanded_dir, regex);
 	}
 	else {		
 					// Clear everything
@@ -643,12 +862,236 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 	}
 }
 
+/*************************************************************************
+	Command processing/printing functions - MountDir list.
+*************************************************************************/
+
+void	SetMountDirList(const ConfigValue &cfg, const string &s, 
+			const MountActionOrderType &mountAction)
+{
+						// Reference counted
+	MountDirItem	str(new MountDirConfig(s, false, false, false,
+					       mountAction));
+
+	MountDirList::iterator iter;
+	for (iter = (allConfig[configProfile].*cfg.cfgMountDirListPtr)->begin(); 
+	     iter != (allConfig[configProfile].*cfg.cfgMountDirListPtr)->end(); ++iter) {
+
+						// Same directory name
+		if ((*iter)->dir == str->dir) {
+			if (**iter != *str) {
+						// Different glob options
+						// Use new options
+				(allConfig[configProfile].*cfg.cfgMountDirListPtr)->erase(iter);
+				(allConfig[configProfile].*cfg.cfgMountDirListPtr)->push_back(str);
+			}
+
+						// Silently ignore
+						// duplicate entry
+			return;
+		}
+	} 
+						// New entry
+	(allConfig[configProfile].*cfg.cfgMountDirListPtr)->push_back(str);
+}
+
+void	ConfigMountDirList(const char *cmd, char *buf, size_t from, size_t len, 
+			   const ConfigValue &cfg)
+{
+	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
+
+	if (buf[from] != '\"')
+		QuoteNotFound(cmd, buf);
+	from++;
+
+					// Shell-style quote seek
+	size_t end_quote = SeekEndQuoteOrThrow(cmd, buf, from, len);
+
+					// Command argument is "" ?	
+	if (from == end_quote)
+		BlankEntry(cmd, buf);
+
+					// Process directory
+	string	dir(buf+from, end_quote-from);
+	string	expanded_dir;
+	try {
+		expanded_dir = ExpandDirectory(dir, true);
+
+		expanded_dir = ResolveSymlink(expanded_dir);
+	}
+	catch (ErrorGenericSyntax &e) {
+		PrintErrorMessage(e.what(), cmd, buf);
+	}
+
+	MountActionOrderType mountAction;
+
+	from = SeekNonSpace(buf, end_quote+1, len);
+	while (!IsEmpty(buf, from, len)) {
+		if (buf[from] != ',')
+			CommaExpected(cmd, buf);
+
+		from = SeekNonSpace(buf, from+1, len);
+		size_t tok_end = SeekConfigTokenEnd(buf, from, len);
+		if (tok_end-from == 4
+			 && CompareStringCase("Skip", buf+from, 4) == 0) {
+			mountAction.push_back(mountActionSkip);
+			from += 4;
+		}
+/* 		else if (tok_end-from == 8
+			 && CompareStringCase("FileList", buf+from, 8) == 0) {
+			mountAction.push_back(mountActionFileList);
+			from += 8;
+		}*/
+		else if (tok_end-from == 3
+			 && CompareStringCase("All", buf+from, 3) == 0) {
+			mountAction.push_back(mountActionAll);
+			from += 7;
+		}
+		else if (tok_end-from == 4
+			 && CompareStringCase("Tree", buf+from, 4) == 0) {
+			mountAction.push_back(mountActionTree);
+			from += 8;
+		}
+		else
+			InvalidOption(cmd, buf);
+	}
+
+	SetMountDirList(cfg, expanded_dir, mountAction);
+}
+
+void	PrintMountDirList(const char *cmd, const ConfigValue &cfg)
+{
+	cout << cmd << ": ";
+
+	bool	indent = false;
+	size_t	num_indent = strlen(cmd)+2;
+
+	for (MountDirList::iterator iter
+	       = (allConfig[configProfile].*cfg.cfgMountDirListPtr)->begin(); 
+	     iter != (allConfig[configProfile].*cfg.cfgMountDirListPtr)->end(); ++iter) {
+		if (indent) {
+			for (size_t i = 0; i < num_indent; ++i)
+				cout << ' ';
+		}
+		else
+			indent = true; 
+		cout << (*iter)->dir;
+
+		bool first = true;
+		const MountActionOrderType &mountAction = (*iter)->mountAction;
+
+		for (MountActionOrderType::const_iterator iter2 = mountAction.begin();
+		     iter2 != mountAction.end(); ++iter2) {
+
+			if (first)
+				cout << ' ';
+
+			switch (*iter2) {
+				case mountActionSkip:
+					cout << " Skip";
+					break;
+				case mountActionAll:
+					cout << " All";
+					break;
+				case mountActionTree:
+					cout << " Tree";
+					break;
+				case mountActionFileList:
+					cout << " FileList";
+					break;
+			}
+
+			first = false;
+		}
+
+		cout << '\n';
+	}
+
+				// List is empty
+	if (! indent)
+		cout << '\n';
+}
+
+/*************************************************************************
+	Command processing/printing functions - Clear MountDir list.
+*************************************************************************/
+
+void	SetClearMountDirList(const ConfigValue &cfg)
+{
+	(allConfig[configProfile].*cfg.cfgMountDirListPtr)->clear();
+}
+
+void	SetClearMountDirList(const ConfigValue &cfg, const string &s, bool regex)
+{
+#ifdef	CLIB_HAVE_REGEX
+	if (regex)
+		(allConfig[configProfile].*cfg.cfgMountDirListPtr)->remove_if(IsRegPatternMatch(s));
+	else
+#endif
+		(allConfig[configProfile].*cfg.cfgMountDirListPtr)->remove_if(IsPatternMatch(s));
+}
+
+void	ConfigClearMountDirList(const char *cmd, char *buf, size_t from, size_t len, 
+			      const ConfigValue &cfg)
+{
+	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
+
+	if (CompareStringCase("all", buf+from, 3)) {
+
+					// Expect directory name
+
+		bool regex = false;
+		if (buf[from] == 'r' || buf[from] == 'R') {
+#ifdef	CLIB_HAVE_REGEX
+			regex = true;
+			from = SeekNonSpace(buf, from+1, len);
+#else
+			NoRegex(cmd, buf);
+#endif
+		}
+
+		if (buf[from] != '\"')
+			QuoteNotFound(cmd, buf);
+		from++;
+
+					// Shell-style quote seek
+		size_t end_quote = SeekEndQuoteOrThrow(cmd, buf, from, len);
+
+		if (!IsEmpty(buf, end_quote+1, len))
+			ExtraChar(cmd, buf);
+
+					// Command argument is "" ?	
+		if (from == end_quote)
+			BlankEntry(cmd, buf);
+
+					// Process directory
+		string	dir(buf+from, end_quote-from);
+		string	expanded_dir;
+		try {
+			expanded_dir = ExpandDirectory(dir, true, regex);
+		}
+		catch (ErrorGenericSyntax &e) {
+			PrintErrorMessage(e.what(), cmd, buf);
+		}
+
+		SetClearMountDirList(cfg, expanded_dir, regex);
+	}
+	else {		
+					// Clear everything
+	
+		if (!IsEmpty(buf, from+3, len))
+			ExtraChar(cmd, buf);
+
+		SetClearMountDirList(cfg);
+	}
+}
+
 /*************************************************************************
 	Command processing/printing functions - Attributes
 *************************************************************************/
 
 struct	AttrColorMap {
-	char		*name;
+	const char	*name;
 	ATTR_TYPE	value;
 };
 
@@ -946,66 +1389,74 @@ struct	ConfigInfo {
 ConfigFunc cfgFuncYesNo = { ConfigYesNo, PrintYesNo };
 ConfigFunc cfgFuncUnsigned = { ConfigUnsigned, PrintUnsigned };
 ConfigFunc cfgFuncKeyBinding = { ConfigKeyBinding, PrintKeyBinding };
+ConfigFunc cfgFuncShowNewDir = { ConfigShowNewDir, PrintShowNewDir };
 ConfigFunc cfgFuncStringList = { ConfigStringList, PrintStringList };
-ConfigFunc cfgFuncStringListNoGlob = { ConfigStringList, PrintStringListNoGlob };
+ConfigFunc cfgFuncStringListNoGlob = { ConfigStringListNoGlob, PrintStringListNoGlob };
 ConfigFunc cfgFuncClearStringList = { ConfigClearStringList, 0 };
+ConfigFunc cfgFuncMountDirList = { ConfigMountDirList, PrintMountDirList };
+ConfigFunc cfgFuncClearMountDirList = { ConfigClearMountDirList, 0 };
 ConfigFunc cfgFuncColorAttr = { ConfigColorAttr, PrintColorAttr };
 ConfigFunc cfgFuncColor = { ConfigColor, PrintColor };
 ConfigFunc cfgFuncBWAttr = { ConfigBWAttr, PrintBWAttr };
 
-#define NUM_CONFIG_INFO	42
+#define NUM_CONFIG_INFO	46
 ConfigInfo configInfo[NUM_CONFIG_INFO] = {
-	{ "QuietFullScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietFull, 0, 0, 0 } },
-	{ "QuietSmartScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietSmart, 0, 0, 0 } },
-	{ "QuietPartialScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietPartial, 0, 0, 0 } },
-	{ "AutoScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgAutoScan, 0, 0, 0 } },
-	{ "SpaceSelect", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSpaceSelect, 0, 0, 0 } },
-	{ "SortTree", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSortTree, 0, 0, 0 } },
-	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgCaseSensitiveSort, 0, 0, 0 } },
-	{ "GraphicChar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGraphicChar, 0, 0, 0 } },
-	{ "ScrollBar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgScrollBar, 0, 0, 0 } },
-	{ "GlobDot", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobDot, 0, 0, 0 } },
-	{ "GlobPath", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobPath, 0, 0, 0 } },
+	{ "QuietFullScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietFull, 0, 0, 0, 0, 0 } },
+	{ "QuietSmartScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietSmart, 0, 0, 0, 0, 0 } },
+	{ "QuietPartialScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietPartial, 0, 0, 0, 0, 0 } },
+	{ "AutoScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgAutoScan, 0, 0, 0, 0, 0 } },
+	{ "SpaceSelect", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSpaceSelect, 0, 0, 0, 0, 0 } },
+	{ "SortTree", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSortTree, 0, 0, 0, 0, 0 } },
+	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgCaseSensitiveSort, 0, 0, 0, 0, 0 } },
+	{ "GraphicChar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGraphicChar, 0, 0, 0, 0, 0 } },
+	{ "ScrollBar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgScrollBar, 0, 0, 0, 0, 0 } },
+	{ "GlobDot", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobDot, 0, 0, 0, 0, 0 } },
+	{ "GlobPath", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobPath, 0, 0, 0, 0, 0 } },
 
-	{ "KeyBinding", &cfgFuncKeyBinding, { 0, 0, 0, 0, 0, &KcdConfig::cfgKey } },
+	{ "KeyBinding", &cfgFuncKeyBinding, { 0, 0, 0, 0, 0, 0, &KcdConfig::cfgKey, 0 } },
+	{ "ShowNewDir", &cfgFuncShowNewDir, { 0, 0, 0, 0, 0, 0, 0, &KcdConfig::cfgShowNewDir } },
 
-	{ "ShowListThreshold", &cfgFuncUnsigned, { &KcdConfig::cfgShowListThreshold, 0, 0, 0, 0, 0 } },
-	{ "FuzzySize", &cfgFuncUnsigned, { &KcdConfig::cfgFuzzySize, 0, 0, 0, 0, 0 } },
+	{ "ShowListThreshold", &cfgFuncUnsigned, { &KcdConfig::cfgShowListThreshold, 0, 0, 0, 0, 0, 0, 0 } },
+	{ "FuzzySize", &cfgFuncUnsigned, { &KcdConfig::cfgFuzzySize, 0, 0, 0, 0, 0, 0, 0 } },
+	{ "MouseScrollRate", &cfgFuncUnsigned, { &KcdConfig::cfgMouseScrollRate, 0, 0, 0, 0, 0, 0, 0 } },
 
-	{ "StartDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0 } },
-	{ "SkipDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0 } },
+	{ "StartDir", &cfgFuncStringListNoGlob, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0, 0, 0 } },
+	{ "SkipDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0, 0, 0 } },
 
-	{ "DefaultTree", &cfgFuncStringListNoGlob, { 0, 0, 0, &KcdConfig::cfgDefaultTree, 0, 0 } },
+	{ "DefaultTree", &cfgFuncStringListNoGlob, { 0, 0, 0, &KcdConfig::cfgDefaultTree, 0, 0, 0, 0 } },
 
-	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0 } },
-	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0 } },
+	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0, 0, 0 } },
+	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0, 0, 0 } },
 
-	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0 } },
-	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0 } },
-	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0 } },
-	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0 } },
-	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0 } },
-	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0 } },
-	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0 } },
-	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0 } },
-	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0 } },
-	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0 } },
+	{ "MountDir", &cfgFuncMountDirList, { 0, 0, 0, 0, &KcdConfig::cfgMountDir, 0, 0, 0 } },
+	{ "ClearMountDir", &cfgFuncClearMountDirList, { 0, 0, 0, 0, &KcdConfig::cfgMountDir, 0, 0, 0 } },
 
-	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0 } },
-	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0 } },
-	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0 } },
-	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0 } },
-	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0 } },
-	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0 } },
+	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0, 0 } },
+	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0, 0 } },
+	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0, 0 } },
+	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0, 0 } },
+	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0, 0 } },
+	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0, 0 } },
+	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0, 0 } },
+	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0, 0 } },
+	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0, 0 } },
+	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0, 0 } },
 
-	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0 } },
-	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0 } },
-	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0 } },
-	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0 } },
-	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0 } },
-	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0 } },
+	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0, 0 } },
+	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0, 0 } },
+	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0, 0 } },
+	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0, 0 } },
+	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0, 0 } },
+	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0, 0 } },
 
-	{ "DefaultBackground", &cfgFuncColor, { 0, &KcdConfig::cfgDefaultBackground, 0, 0, 0, 0 } }
+	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0, 0 } },
+	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0, 0 } },
+	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0, 0 } },
+	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0, 0 } },
+	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0, 0 } },
+	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0, 0 } },
+
+	{ "DefaultBackground", &cfgFuncColor, { 0, &KcdConfig::cfgDefaultBackground, 0, 0, 0, 0, 0, 0 } }
 };
 
 /*************************************************************************
@@ -1194,7 +1645,9 @@ void	LoadAllConfig(bool quiet)
 
 			// Transfer all config to relevant variables
 	isACSFallBack = !kcdConfig.cfgGraphicChar;
+	
 	khGeometryManager::disableScrollBar = !kcdConfig.cfgScrollBar;
+	NCScreenManager::SetMouseScrollRate(kcdConfig.cfgMouseScrollRate);
 }
 
 /*************************************************************************
@@ -1211,6 +1664,7 @@ void	PrintConfig()
 	PrintProfile();
 	SetConfigProfile(GetCurrentProfile());
 	for (int i = 0; i < NUM_CONFIG_INFO; ++i) {
+		// Ignore attribute and color handled in PrintAttrConfig
 		if (configInfo[i].cfgFunc->cfgPrintFunc && !configInfo[i].cfgValue.cfgAttrPtr
 		    && configInfo[i].cfgFunc->cfgPrintFunc != PrintColor)
 			configInfo[i].cfgFunc->cfgPrintFunc(configInfo[i].cfgName,
