@@ -1,6 +1,6 @@
 /*
 	confobj.h	Configuration Class Header
-	Copyright (c) 2000, 2001, 2002, 2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -26,29 +26,79 @@
 #include "list.h"
 #include "gzfileio.h"
 
+#ifdef	CLIB_HAVE_REGEX
+# include <sys/types.h>
+# include <regex.h>
+int k_regcomp(regex_t *r, const string &s1, int flags);
+#endif
+
+#include CXX__HEADER_climits
 #include CXX__HEADER_map
+#include CXX__HEADER_vector
 
 USING_NAMESPACE_STD;
 
+enum MountActionType {
+	mountActionSkip = 0,
+	mountActionAll,
+	mountActionTree,
+	mountActionFileList
+};
+
+typedef vector<MountActionType> MountActionOrderType;
+
+enum ShowNewDirType {
+	showNewDirNo = 0,
+	showNewDirYes,
+	showNewDirMulti
+};
+
 struct DirConfig {
 	string	dir;
+	bool	regex;
+
 	bool	globDot;
 	bool	globPath;
 
-	DirConfig(string dir_, bool globDot_, bool globPath_) : 
-		dir(dir_), globDot(globDot_), globPath(globPath_) {}
+#ifdef	CLIB_HAVE_REGEX
+	regex_t	regex_buffer;
+#endif
 
-	bool	operator ==(const DirConfig& d) const {
-		return dir == d.dir && globDot == d.globDot
+	DirConfig(string dir_, bool regex_, bool globDot_, bool globPath_) : 
+		dir(dir_), regex(regex_), globDot(globDot_), globPath(globPath_) {
+
+#ifdef	CLIB_HAVE_REGEX
+		k_regcomp(&regex_buffer, dir, REG_EXTENDED);
+#endif
+	}
+
+	virtual bool	operator ==(const DirConfig& d) const {
+		return dir == d.dir && regex == d.regex
+				    && globDot == d.globDot
 				    && globPath == d.globPath;
 	}
-	bool	operator !=(const DirConfig& d) const {
+	virtual bool	operator !=(const DirConfig& d) const {
 		return !(*this == d);
 	}
+	virtual ~DirConfig() {}
+};
+
+struct MountDirConfig : public DirConfig {
+	MountActionOrderType mountAction;
+
+	MountDirConfig(string dir_, bool regex_, bool globDot_,
+		       bool globPath_, const MountActionOrderType &mountAction_) :
+		DirConfig(dir_, regex_, globDot_, globPath_), mountAction(mountAction_) {}
+	virtual bool	operator ==(const MountDirConfig& d) const {
+		return this->DirConfig::operator ==(d)
+			&& mountAction == d.mountAction;
+	}
 };
 
 typedef sptr_list<DirConfig>	DirList;
 typedef sptr<DirConfig>		DirItem;
+typedef sptr_list<MountDirConfig>	MountDirList;
+typedef sptr<MountDirConfig>		MountDirItem;
 
 #define SCAN_MODE_FULL		0
 #define SCAN_MODE_SMART		1
@@ -90,6 +140,9 @@ struct KcdConfig {
 
 	unsigned	cfgShowListThreshold;
 	unsigned	cfgFuzzySize;
+	unsigned	cfgMouseScrollRate;
+
+	ShowNewDirType	cfgShowNewDir;
 
 	AttrConfig	cfgAttrNormal;
 	AttrConfig	cfgAttrLink;
@@ -115,6 +168,8 @@ struct KcdConfig {
 
 	DirList *cfgDefaultTree;
 
+	MountDirList *cfgMountDir;
+
 	KcdConfig();
 	~KcdConfig();
 
@@ -139,6 +194,7 @@ extern configMap allConfig;
 
 bool	FindSkipDir(const string &str);
 bool	FindStartDir(const string &str);
+bool	FindMountDir(const string &str);
 
 int	k_fnmatch(const string &pattern, const string &str);
 
