@@ -1,6 +1,6 @@
 /*
 	cstrlib.cc	Add C++ String Handling To C Library Functions
-	Copyright (c) 2000, 2001, 2004 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2004, 2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -51,7 +51,7 @@ struct passwd *k_getpwnam(const string &name)
 	return getpwnam(name.c_str());
 }
 
-FILE *k_fopen(const string &path, char *mode)
+FILE *k_fopen(const string &path, const char *mode)
 {
 	return fopen(path.c_str(), mode);
 }
