@@ -1,6 +1,6 @@
 /*
 	cxxlib.h	Correct C++ Library Problems Header
-	Copyright (c) 2000,2001,2002 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2002,2007 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -27,19 +27,6 @@
 
 USING_NAMESPACE_STD;
 
-void dump_core();
-
-//
-// new/delete operators
-//
-
-void *operator new (size_t s) CXX__NEW_THROW;
-void operator delete (void *p) CXX__DELETE_THROW;
-#ifdef CXX__HAVE_ARRAY_NEW
-void *operator new[] (size_t s) CXX__NEW_THROW;
-void operator delete[] (void *p) CXX__DELETE_THROW;
-#endif	/* CXX__HAVE_ARRAY_NEW */
-
 //
 // string I/O stream
 //
@@ -69,10 +56,4 @@ template<class T> T Max(T a, T b)
 	return a;
 }
 
-//
-// Run-time setup
-//
-
-void	cxxlib();
-
 #endif	/* __K_CXXLIB_H */
