@@ -1,6 +1,6 @@
 /*
 	dirutil.h	Directory Utility Functions Header
-	Copyright (c) 2000, 2001,2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2003, 2004, 2005 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,8 @@
 
 USING_NAMESPACE_STD;
 
-string	ExpandDirectory(const string &src, bool quote = false);
+string	ResolveSymlink(const string &src);
+string	RelativeToAbsolutePath(const string &path, const string &cwd);
+string	ExpandDirectory(const string &src, bool quote = false, bool regex = false);
 
 #endif	/* __K_DIRUTIL_H */
