@@ -1,6 +1,6 @@
 /*
 	gentree.cc	Generate Directory Tree For Display
-	Copyright (c) 2000, 2002, 2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002, 2003, 2004, 2005, 2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -59,12 +59,14 @@ string	html_skipped;
 string	html_outside_tree;
 string	html_not_in_file;
 string	html_not_found;
+string	html_no_filelist;
 
 size_t	html_unreadable_width;
 size_t	html_skipped_width;
 size_t	html_outside_tree_width;
 size_t	html_not_in_file_width;
 size_t	html_not_found_width;
+size_t	html_no_filelist_width;
 
 void	GenerateHTMLStrings()
 {
@@ -73,12 +75,14 @@ void	GenerateHTMLStrings()
 	html_outside_tree = _(" [outside tree]");
 	html_not_in_file = _(" [*]");
 	html_not_found = _(" [not found]");
+	html_no_filelist = _(" [no filelist]");
 
 	html_unreadable_width = GetStringWidth(html_unreadable);
 	html_skipped_width = GetStringWidth(html_skipped);
 	html_outside_tree_width = GetStringWidth(html_outside_tree);
 	html_not_in_file_width = GetStringWidth(html_not_in_file);
 	html_not_found_width = GetStringWidth(html_not_found);
+	html_no_filelist_width = GetStringWidth(html_no_filelist);
 }
 
 /*************************************************************************
@@ -244,7 +248,12 @@ void	HyperTreeDraw::RecursiveGenerateHTML(HyperDocument &html, WINDOW *pad,
 					     size_t pos, const string &current_dir)
 {
 	if (d->isSkip) {
-		if (d->isSkip == 2) {
+		if (d->isSkip == 3) {
+			html.DrawString(pad, html_no_filelist);
+			curCol += html_no_filelist_width;
+			pos += html_no_filelist_width;
+		}
+		else if (d->isSkip == 2) {
 			html.DrawString(pad, html_not_in_file);
 			curCol += html_not_in_file_width;
 			pos += html_not_in_file_width;
@@ -255,6 +264,7 @@ void	HyperTreeDraw::RecursiveGenerateHTML(HyperDocument &html, WINDOW *pad,
 			pos += html_skipped_width;
 		}
 	}
+
 	if (d->isUnreadable) {
 		html.DrawString(pad, html_unreadable);
 		curCol += html_unreadable_width;
@@ -472,12 +482,13 @@ struct MatchInfoIsNotBookmarkedNorCurDir {
 	Find Match
 *************************************************************************/
 
-// The add_curdir feature isn't handled here fuzzy find because it
+// The add_curdir feature isn't handled in fuzzy find because it
 // doesn't work well with error sorting order.
-void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
-				  sptr<DirectoryEntry> &dir, 
-				  const string &str,
-				  const string &current_dir)
+void	FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
+				   sptr<DirectoryEntry> &dir, 
+				   const string &str,
+				   regex_t *reg,
+				   const string &current_dir)
 {
 	unsigned subStrLen;
 	unsigned StrLen;
@@ -485,7 +496,7 @@ void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
 	unsigned TmpErr;
 	int TmpVal;
 
-	subStrLen=str.size();
+	subStrLen = str.size();
 	if (dir->IsDir() && dir->isUnreadable != 1) {
 		string mstr = dir->GetNameStr();
 		StrLen = mstr.size();
@@ -495,7 +506,7 @@ void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
 		if (subStrLen < StrLen) {
 			TotCnt = StrLen-subStrLen+1;
 			unsigned err = static_cast<unsigned>(-1);
-			for (cnt1=0; cnt1 < TotCnt; cnt1++) {
+			for (cnt1 = 0; cnt1 < TotCnt; cnt1++) {
 				TmpErr = 0;
 				for (cnt2 = 0; cnt2 < subStrLen; cnt2++) {
 					TmpVal = mstr[cnt1+cnt2-1]
@@ -522,6 +533,7 @@ void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
 					if ((*iter).error > err)
 						break;
 				}
+//FIXME: Add regex matching here
 					// Insert before iter
 				match_list.insert(iter,
 						  MatchInfo(current_dir, dir, false, err));
@@ -535,10 +547,10 @@ void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
 					if ((*iter).error > err)
 						break;
 				}
+//FIXME: Add regex matching here
 					// Insert before iter
 				match_list.insert(iter,
 						  MatchInfo(current_dir, dir, false, err));
-//cout << current_dir << ' ' << err << '\n';
 			}
 		}
 	}
@@ -553,22 +565,24 @@ void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
 				new_dir += '/';
 			new_dir += (*iter)->GetNameStr();
 
-			FuzzyRecursiveFindMatchDir(match_list, *iter, str, new_dir);
+			FuzzyRecursiveFindMatchDir(match_list, *iter,
+						   str, reg, new_dir);
 		}
 	}
 }
 
-size_t FuzzyFindMatchDir(list<MatchInfo> &match_list, const string &linktext)
+size_t	FuzzyFindMatchDir(list<MatchInfo> &match_list, const string &str,
+			  regex_t *reg)
 {
 	match_list.clear();
 
-	if (linktext.size() == 0)		// In case getcwd fails
+	if (str.size() == 0)			// In case getcwd fails
 		return 0;
 
 	for (sptr_list<DirectoryEntry>::iterator iter = dirTree.begin();
 	     iter!=dirTree.end(); ++iter) {
 		string new_dir = (*iter)->GetNameStr();
-		FuzzyRecursiveFindMatchDir(match_list, *iter, linktext,
+		FuzzyRecursiveFindMatchDir(match_list, *iter, str, reg,
 					   new_dir);
 	}
 
@@ -576,13 +590,15 @@ size_t FuzzyFindMatchDir(list<MatchInfo> &match_list, const string &linktext)
 }
 
 void	RecursiveFindMatchDir(list<MatchInfo> &match_list, sptr<DirectoryEntry> &dir, 
-			      const string &str, const string &current_dir,
+			      const string &str, regex_t *reg,
+			      const string &current_dir,
 			      bool add_curdir)
 {
 	if (dir->IsDir() && dir->isUnreadable != 1 &&
 	    dir->GetNameStr().find(str) != string::npos) {	// Found
 
-		match_list.push_back(MatchInfo(current_dir, dir, false));
+		if (MatchPathRegex(reg, current_dir))
+			match_list.push_back(MatchInfo(current_dir, dir, false));
 	}
 	else if (add_curdir && saveCwd.size() && current_dir == saveCwd) {
 							// Write current directory
@@ -600,76 +616,35 @@ void	RecursiveFindMatchDir(list<MatchInfo> &match_list, sptr<DirectoryEntry> &di
 				new_dir += '/';
 			new_dir += (*iter)->GetNameStr();
 			
-			RecursiveFindMatchDir(match_list, *iter, str, new_dir,
-					      add_curdir);
+			RecursiveFindMatchDir(match_list, *iter, str, reg,
+					      new_dir, add_curdir);
 		}
 	}
 }
 
-size_t	FindMatchDir(list<MatchInfo> &match_list, const string &linktext,
-		     bool add_curdir = false)
+size_t	FindMatchDir(list<MatchInfo> &match_list, const string &str,
+		     regex_t *reg, bool add_curdir = false)
 {
 	match_list.clear();	
 	
-	if (linktext.size() == 0)		// In case getcwd fails
+	if (str.size() == 0)		// In case getcwd fails
 		return 0;
 		
 	for (sptr_list<DirectoryEntry>::iterator iter = dirTree.begin();
 	     iter != dirTree.end(); ++iter) {
 		string new_dir = (*iter)->GetNameStr();
-		RecursiveFindMatchDir(match_list, *iter, linktext, new_dir,
+		RecursiveFindMatchDir(match_list, *iter, str, reg, new_dir,
 				      add_curdir);
 	}
 	
 	if (kcdConfig.cfgFuzzySize &&
 	    (match_list.size() == 0
-	     || match_list.size() == 1 && match_list.front().is_curdir == true))
-		return FuzzyFindMatchDir(match_list, linktext);
+	     || (match_list.size() == 1 && match_list.front().is_curdir == true)))
+		return FuzzyFindMatchDir(match_list, str, reg);
 
 	return match_list.size();
 }
 
-/*************************************************************************
-	Find Bookmark
-*************************************************************************/
-
-void	RecursiveFindBookmarkedDir(list<MatchInfo> &match_list, sptr<DirectoryEntry> &dir, 
-				    const string &current_dir)
-{
-	if (dir->IsDir() && dir->isUnreadable != 1 &&
-	    dir->bookmarked) {			// Found
-
-		match_list.push_back(MatchInfo(current_dir, dir, false));
-	}
-
-	for (sptr_list<DirectoryEntry>::iterator iter = dir->subDir.begin();
-	     iter != dir->subDir.end(); ++iter) {
-
-			    			// For dirs
-		if ((*iter)->IsDir()) {
-			string new_dir = current_dir;
-			if (new_dir[new_dir.size()-1] != '/')
-				new_dir += '/';
-			new_dir += (*iter)->GetNameStr();
-			
-			RecursiveFindBookmarkedDir(match_list, *iter, new_dir);
-		}
-	}
-}
-
-size_t	FindBookmarkedDir(list<MatchInfo> &match_list)
-{
-	match_list.clear();	
-	
-	for (sptr_list<DirectoryEntry>::iterator iter = dirTree.begin();
-	     iter != dirTree.end(); ++iter) {
-		string new_dir = (*iter)->GetNameStr();
-		RecursiveFindBookmarkedDir(match_list, *iter, new_dir);
-	}
-	
-	return match_list.size();
-}
-
 /*************************************************************************
 	Generate HTML-like codes from the match list
 *************************************************************************/
@@ -1056,22 +1031,78 @@ void FullScreenMode(HyperDraw *drawObj, const string &hyperInitPos)
 	}
 }
 
+/*************************************************************************
+	MountDir handling
+*************************************************************************/
+
+void	ProcessMountDir(bool search, const string &root)
+{
+	for (MountDirList::iterator iter = kcdConfig.cfgMountDir->begin();
+	     iter != kcdConfig.cfgMountDir->end(); ++iter) {
+		FindDirInfo *f = FindDir(&dirTree, (*iter)->dir, false);
+		if (f) {
+			f->dir_ptr->subDir.clear();
+
+			if (root.size()) {
+				if ((*iter)->dir.size() < root.size()
+				    || (*iter)->dir.substr(0, root.size())
+					!= root)
+					continue;
+			}
+
+			f->dir_ptr->isSkip = 0;
+
+				// This make sure the MountDir
+				// directory is actually scanned
+				// and not skipped
+			MountDirList* m = kcdConfig.cfgMountDir;
+			kcdConfig.cfgMountDir = new MountDirList;
+
+			const MountActionOrderType &mountAction = (*iter)->mountAction;
+
+			for (MountActionOrderType::const_iterator iter2 = mountAction.begin();
+			     iter2 != mountAction.end(); ++iter2) {
+
+				bool result = false;
+				switch (*iter2) {
+					case mountActionSkip:
+						result = true;
+						break;
+					case mountActionAll:
+						MountDirScanDir((*iter)->dir);
+						result = true;
+						break;
+					case mountActionTree:
+						if (search)
+							;
+						else {
+							MountDirScanDir((*iter)->dir);
+							result = true;
+						}
+						break;
+					case mountActionFileList:
+						break;
+				}
+
+				if (result)
+					break;
+			}
+
+			delete kcdConfig.cfgMountDir;
+			kcdConfig.cfgMountDir = m;
+		}
+		delete f;
+	}
+}
+
 /*************************************************************************
 	Display tree functions
 *************************************************************************/
 
 void AddDisplayTree(const string &root, sptr_list<DirectoryEntry> *tree)
 {
-	string root_abs;			// Convert relative to absolute
-						// directory name
-	if (root[0] == '/' || root[0] == '~')
-		root_abs = ExpandDirectory(root, false);
-	else {
-		root_abs = saveCwd;
-		root_abs += '/';
-		root_abs += root;
-		root_abs = ExpandDirectory(root_abs, false);
-	}
+	string root_abs = RelativeToAbsolutePath(root, saveCwd);
+	root_abs = ExpandDirectory(root_abs, false);
 
 						// Allocate from heap since
 						// its lifetime is longer than
@@ -1126,15 +1157,19 @@ HyperDraw *GetDisplayTree(const string &root)
 	return drawObj;
 }
 
-void	DisplayTree(const string &root, bool rescan)
+void	DisplayTree(const string &root, bool use_mount_dir, bool rescan)
 {
-	string str = root;
-	if (str.size() == 0) {
+	string str;
+	if (root.size() == 0) {
 		if (saveCwd.size())
 			str = saveCwd;
 		else
 			throw ErrorGenericCommandLine(_("cannot determine current directory"));
 	}
+	else {
+		str = RelativeToAbsolutePath(root, saveCwd);
+		str = ExpandDirectory(str, false);
+	}
 
 	FindDirInfo *find = FindDir(&dirTree, str, false, false);
 	if (!find)
@@ -1142,15 +1177,23 @@ void	DisplayTree(const string &root, bool rescan)
 
 	if (rescan)
 		ScanDirAndDisplay(str, false);
+
+	if (use_mount_dir)
+		ProcessMountDir(false, root);
+
 	GenerateHTMLStrings();
 	HyperDraw *drawObj = GetDisplayTree(str);
 	FullScreenMode(drawObj, "");
 }
 
-void	DisplayFullTree(bool rescan)
+void	DisplayFullTree(bool use_mount_dir, bool rescan)
 {
 	if (rescan)
 		ScanDir(false);
+
+	if (use_mount_dir)
+		ProcessMountDir(false, "");
+
 	GenerateHTMLStrings();
 	HyperDraw *drawObj = new HyperTreeDraw(&dirTree);
 	FullScreenMode(drawObj, "");
@@ -1160,7 +1203,25 @@ void	DisplayFullTree(bool rescan)
 	Change directory
 *************************************************************************/
 
-void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext);
+void	GetBookmarks(list<MatchInfo> &match_list, const string &str,
+		     regex_t *reg);
+
+void	CheckCDString(const string &str)
+{
+	if (str.size()) {
+#ifdef	CLIB_HAVE_REGEX
+		if (str[0] == '/')
+			throw ErrorGenericCommandLine(_("invalid search string"));
+		if (str[str.size()-1] == '/')
+			throw ErrorGenericCommandLine(_("invalid search string"));
+		if (str.find("///") != string::npos)
+			throw ErrorGenericCommandLine(_("invalid search string"));
+#else
+		if (str.find('/'))
+			throw ErrorGenericCommandLine(_("regular expression not supported"));
+#endif
+	}
+}
 
 // FIXME: init_pos and adjust look redundant
 
@@ -1229,7 +1290,7 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 				}
 				else
 					throw ErrorGenericFile(_("cannot find directory "
-								 "containing the string %$"),
+								 "matching the string %$"),
 							       str);
 			}
 		}
@@ -1251,14 +1312,33 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 	k_fputs((*iter).name, file);
 	fclose(file);
 
-	if (match_list.size() > adjust+1)	// More than one matched dir
+	bool show_new = false;
+	switch (kcdConfig.cfgShowNewDir) {
+		case showNewDirNo:
+			break;
+		case showNewDirYes:
+			show_new = true;
+			break;
+		case showNewDirMulti:
+							// More than one matched dir
+			if (match_list.size() > adjust+1)
+				show_new = true;
+			break;
+	}
+	if (show_new)
 		cout << _("change to ") << (*iter).name << '\n';
 }
 
-void	CD(const string &linktext, bool entire_tree, size_t match_no)
+void	CD(const string &linktext, bool entire_tree, 
+	   bool use_mount_dir, size_t match_no)
 {
+	CheckCDString(linktext);
+
 	LoadDirFile(true);
 
+	if (use_mount_dir)
+		ProcessMountDir(linktext.size(), "");
+
 	HyperDraw *drawObj = 0;
 	list<MatchInfo>	match_list;			// Must match lifetime
 							// of drawObj!
@@ -1271,6 +1351,8 @@ void	CD(const string &linktext, bool entire_tree, size_t match_no)
 	GenerateHTMLStrings();
 
 	if (linktext.size()) {				// kcd DIR
+		string	str = MakeString(linktext);
+		regex_t *reg = MakePathRegex(linktext);
 
 		ImportBookmarks("");
 
@@ -1280,10 +1362,10 @@ void	CD(const string &linktext, bool entire_tree, size_t match_no)
 							// to aid cycling through
 							// all match directory
 							// in the command line
-			FindMatchDir(match_list, linktext, true);
+			FindMatchDir(match_list, str, reg, true);
 
 		else {
-			GetBookmarks(match_list, linktext);
+			GetBookmarks(match_list, str, reg);
 			if (match_list.size()) {
 				if (match_list.size() > 1 && saveCwd.size()) {
 					list<MatchInfo>::iterator iter;
@@ -1300,7 +1382,7 @@ void	CD(const string &linktext, bool entire_tree, size_t match_no)
 				}
 			}
 			else {
-				FindMatchDir(match_list, linktext, true);
+				FindMatchDir(match_list, str, reg, true);
 			}
 		}
 
@@ -1417,22 +1499,15 @@ void	RemoveBookmark(const string &str, bool verify = true)
 	remove(bookmark_list.begin(), bookmark_list.end(), s);
 }
 
-void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext)
+void	GetBookmarks(list<MatchInfo> &match_list, const string &str,
+		     regex_t *reg)
 {
 	for (list<string>::const_iterator iter = bookmark_list.begin();
 	     iter != bookmark_list.end(); iter++) {
 
-		if (linktext.size()) {
-							// Search string after the
-							// last '/'
-			size_t slash_pos = iter->rfind('/');
-			size_t link_pos;
-			if (slash_pos != string::npos)
-				link_pos = iter->find(linktext, slash_pos+1);
-			else
-				link_pos = iter->find(linktext);
-
-			if (link_pos != string::npos) {
+		if (str.size()) {
+			if (MatchStr(str, *iter)
+			    && MatchPathRegex(reg, *iter)) {
 				FindDirInfo *f = FindDir(&dirTree, *iter, true, false);
 				if (f)
 					match_list.push_back(MatchInfo(*iter, f->dir_ptr, false));
@@ -1498,6 +1573,8 @@ void	ImportBookmarks(const string &filename)
 			AddBookmark(UnquoteShellChars(buffer), false);
 		}
 		catch (ErrorRange &) {
+			// Continue next line without abort
+			// FIXME: Detect UnquoteShellChars errors
 		}
 	}
 }
@@ -1514,7 +1591,14 @@ void	ShowBookmarks(const string &linktext)
 	ImportBookmarks("");
 
 	list<MatchInfo>	match_list;
-	GetBookmarks(match_list, linktext);
+
+	if (linktext.size()) {				// kcd DIR
+		string	str = MakeString(linktext);
+		regex_t *reg = MakePathRegex(linktext);
+		GetBookmarks(match_list, str, reg);
+	}
+	else
+		GetBookmarks(match_list, linktext, 0);
 
 	if (!match_list.size()) {
 		cout << flush;			// Avoid out-of-order display
@@ -1551,22 +1635,17 @@ void	AddBookmarks(int argc, char *argv[], int argcBegin)
 			if (!argv[i][0])
 				continue;
 
-			if (argv[i][0] != '/' && argv[i][0] != '~') {
-				if (saveCwd.size()) {
-					string dir = saveCwd;
-					dir += '/';
-					dir += argv[i];
-					AddBookmark(dir, true);
-				}
-				else {
-					cout << flush;			// Avoid out-of-order display
-					gtout(cerr, _("%$: cannot determine current directory for path %$\n"
-						      "        Directory ignored"))
-					     << progName << argv[i];
-				}
+			if (argv[i][0] != '/' && argv[i][0] != '~'
+			    && !saveCwd.size()) {
+				cout << flush;			// Avoid out-of-order display
+				gtout(cerr, _("%$: cannot determine current directory for path %$\n"
+	 				      "        Directory ignored"))
+				     << progName << argv[i];
+			}
+			else {
+				string dir = RelativeToAbsolutePath(argv[i], saveCwd);
+				AddBookmark(dir, true);
 			}
-			else
-				AddBookmark(argv[i], true);
 		}
 	}
 
@@ -1594,22 +1673,17 @@ void	RemoveBookmarks(int argc, char *argv[], int argcBegin)
 			if (!argv[i][0])
 				continue;
 
-			if (argv[i][0] != '/' && argv[i][0] != '~') {
-				if (saveCwd.size()) {
-					string dir = saveCwd;
-					dir += '/';
-					dir += argv[i];
-					RemoveBookmark(dir, true);
-				}
-				else {
-					cout << flush;			// Avoid out-of-order display
-					gtout(cerr, _("%$: cannot determine current directory for path %$\n"
-						      "        Directory ignored"))
-					     << progName << argv[i];
-				}
+			if (argv[i][0] != '/' && argv[i][0] != '~'
+			    && !saveCwd.size()) {
+				cout << flush;			// Avoid out-of-order display
+				gtout(cerr, _("%$: cannot determine current directory for path %$\n"
+					      "        Directory ignored"))
+				     << progName << argv[i];
+			}
+			else {
+				string dir = RelativeToAbsolutePath(argv[i], saveCwd);
+				AddBookmark(dir, true);
 			}
-			else
-				RemoveBookmark(argv[i], true);
 		}
 	}
 
