@@ -1,6 +1,6 @@
 /*
 	gentree.h	Generate Directory Tree For Display Header
-	Copyright (c) 2000, 2002, 2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002, 2003, 2005 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -44,10 +44,12 @@ FindDirInfo *FindDir(sptr_list<DirectoryEntry> *dirTree, const string &str,
 		     bool honor_cfg, bool change_top, 
 		     bool change_sub);
 
-void	DisplayTree(const string &root, bool rescan = false);
-void	DisplayFullTree(bool rescan = false);
+void	DisplayTree(const string &root, bool use_mount_dir = true,
+		    bool rescan = false);
+void	DisplayFullTree(bool use_mount_dir = true, bool rescan = false);
 
-void	CD(const string &linktext, bool entire_tree = false, size_t match_no = 0);
+void	CD(const string &linktext, bool entire_tree = false, 
+	   bool use_mount_dir = true, size_t match_no = 0);
 void	ShowBookmarks(const string &linktext);
 
 void	AddBookmarks(int argc, char *argv[], int argcBegin);
