@@ -1,6 +1,6 @@
 /*
-	gtstring.cc	C++ Stream Supporting gettext Usage
-	Copyright (c) 2000, 2001, 2002 Kriang Lerdsuwanakij
+	gtstream.cc	C++ Stream Supporting gettext Usage
+	Copyright (c) 2000, 2001, 2002, 2005 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
