@@ -1,6 +1,6 @@
 /*
 	kcdmain.cc	Kriang's Change Directory
-	Copyright (c) 1996-9,2000-2,2003 Kriang Lerdsuwanakij
+	Copyright (c) 1996-9,2000-4,2005,2007,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -69,7 +69,7 @@ void	PrintBanner()
 
 	cout << progName << ' ' << version
 	     << _(" - Kriang's Change Directory\n");
-	gtout(cout, _("(c) 1996-9,2000-2,2003 Kriang Lerdsuwanakij    "
+	gtout(cout, _("(c) 1996-9,2000-5,2007,2009 Kriang Lerdsuwanakij    "
 		      "Release date: %$\n\n"))
 	     << my_strftime(_("%b %e, %Y"), &rel_time);
 
@@ -102,8 +102,10 @@ void	Help()
 		  "     --features     Display features included during compilation\n"
 		  "     -h, --help     Display this help\n"
 		  "     --helpinst     Display help about shell alias/function installation\n");
+	cout << _("     -M             Don't scan directories given by MountDir\n");
 	cout << _("     -p NAME        Use configuration profile NAME.  If used, this must be\n"
-		  "                    the first option in the command line\n");
+		  "                    the first option in the command line\n"
+		  "     --profiles     Display the list of configuration profile\n");
 	cout << _("     -r             Rescan directory tree\n"
 		  "     -rf            Rescan directory tree using full rescan mode\n"
 		  "     -rp DIR        Rescan part of directory tree beginning at DIR\n"
@@ -176,6 +178,28 @@ void	Features()
 #endif
 	     << "\n";
 	
+	cout << _("     regular expression  : ")
+#ifdef	CLIB_HAVE_REGEX
+	     << _("Yes")
+#else
+	     << _("No")
+#endif
+	     << "\n";
+	
+	cout << '\n';
+}
+
+void	Profiles()
+{
+	PrintBanner();
+
+	cout << _("Profiles:\n");
+
+	for (configMap::iterator iter = allConfig.begin();
+	     iter != allConfig.end();
+	     ++iter) {
+		cout << "     " << iter->first << '\n';
+	}
 	cout << '\n';
 }
 
@@ -296,6 +320,7 @@ void	mainReal(int argc, char *argv[])
 		bool	rescan = false;
 		bool	entire_tree = false;
 		bool	forceFullScan = false;
+		bool	use_mount_dir = true;
 
 		vector<string>	partial_dir;
 
@@ -326,6 +351,11 @@ void	mainReal(int argc, char *argv[])
 				Features();
 				return;
 			}
+			else if (strcmp(argv[argcCur], "--profiles") == 0) {
+				LoadAllConfig(false);
+				Profiles();
+				return;
+			}
 			else if (strcmp(argv[argcCur], "--config") == 0) {
 				LoadAllConfig(false);
 				PrintConfig();
@@ -424,8 +454,11 @@ void	mainReal(int argc, char *argv[])
 			else if (strcmp(argv[argcCur], "-p") == 0) {
 				if (argcCur != 1)
 					throw ErrorGenericCommandLine(_("`-p\' must be the first option used"));
-				else if (argcLeft >= 2)
+				else if (argcLeft >= 2) {
+					if (strcmp(argv[argcCur+1], "*") == 0)
+						throw ErrorGenericCommandLine(_("`*\' profile not allowed in command line"));
 					SetCurrentProfile(argv[argcCur+1]);
+				}
 				else
 					throw ErrorGenericCommandLine(_("one argument required for `-p\'"));
 						// Consume argument of -rp
@@ -459,6 +492,10 @@ void	mainReal(int argc, char *argv[])
 						// Use entire tree
 			else if (strcmp(argv[argcCur], "-B") == 0) {
 				entire_tree = true;
+			}
+						// Use entire tree
+			else if (strcmp(argv[argcCur], "-M") == 0) {
+				use_mount_dir = false;
 			}
 						// Display tree
 			else if (strcmp(argv[argcCur], "-t") == 0) {
@@ -468,11 +505,11 @@ void	mainReal(int argc, char *argv[])
 					throw ErrorGenericCommandLine(_("too many arguments for `-t\'"));
 				else if (argcLeft == 2) {
 					LoadDirFile(true);
-					DisplayTree(argv[argcCur+1]);
+					DisplayTree(argv[argcCur+1], use_mount_dir);
 				}
 				else {
 					LoadDirFile(true);
-					DisplayTree("");
+					DisplayTree("", use_mount_dir);
 				}
 				return;
 			}
@@ -486,11 +523,11 @@ void	mainReal(int argc, char *argv[])
 					throw ErrorGenericCommandLine(_("too many arguments for `-tr\'"));
 				else if (argcLeft == 2) {
 					LoadDirFile(true);
-					DisplayTree(argv[argcCur+1], true);
+					DisplayTree(argv[argcCur+1], use_mount_dir, true);
 				}
 				else {
 					LoadDirFile(true);
-					DisplayTree("", true);
+					DisplayTree("", use_mount_dir, true);
 				}
 				return;
 			}
@@ -501,7 +538,7 @@ void	mainReal(int argc, char *argv[])
 				if (argcLeft >= 2)
 					throw ErrorGenericCommandLine(_("argument not allowed for `-T\'"));
 				LoadDirFile(true);
-				DisplayFullTree();
+				DisplayFullTree(use_mount_dir);
 				return;
 			}
 						// Rescan dir.
@@ -555,13 +592,13 @@ void	mainReal(int argc, char *argv[])
 				SetSaveFile();
 
 				if (argcLeft == 1 && rescan == false)
-					CD(argv[argcCur], entire_tree);
+					CD(argv[argcCur], entire_tree, use_mount_dir);
 				else if (argcLeft == 2 && rescan == false) {
 					try {
 						size_t u = StringToUnsigned(argv[argcCur+1], 0);
 						if (u == 0)
 							throw ErrorRange();
-						CD(argv[argcCur], entire_tree, u);
+						CD(argv[argcCur], entire_tree, use_mount_dir, u);
 					}
 					catch (ErrorRange &) {
 						throw ErrorGenericCommandLine(_("invalid match number"));
@@ -588,20 +625,18 @@ void	mainReal(int argc, char *argv[])
 							// no action specified yet
 			SetSaveFile();
 
-			CD("", entire_tree);		// Show entire tree
+			CD("", entire_tree, use_mount_dir);		// Show entire tree
 		}
 	}
 }
 
 int	main(int argc, char *argv[])
 {
-	cxxlib();
-
 	try {
 		// Work around for gcc EH optimization bugs
 		mainReal(argc, argv);
 	}
-	catch(BAD_ALLOC &) {
+	catch(bad_alloc &) {
 		cout << flush;			// Avoid out-of-order display
 		cerr << progName << _(": not enough memory\n");
 		return RET_MEMERR;
