@@ -1,6 +1,6 @@
 /*
 	khdoc.cc	Hypertext Document Classes
-	Copyright (c) 1996-9,2000-2, 2003 Kriang Lerdsuwanakij
+	Copyright (c) 1996-9,2000-2,2003,2007,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -85,7 +85,9 @@
 
 #ifdef USE_UTF8_MODE
 # ifdef HAVE_WCHAR_H
-#  define __USE_XOPEN
+#  ifndef __USE_XOPEN
+#   define __USE_XOPEN
+#  endif
 #  include CXX__HEADER_cwchar
 #  include CXX__HEADER_cwctype
 # endif
@@ -435,7 +437,7 @@ void	HyperDocument::ReadFile(gzFile file)
 							// below
 	bufferSize = ALLOCSIZE;
 	if (buffer == NULL)
-		throw BAD_ALLOC();
+		throw bad_alloc();
 
 	while((byteRead = gzread(file, buffer+bufferPtr, readSize)) ==
 	      static_cast<int>(readSize)) {
@@ -446,7 +448,7 @@ void	HyperDocument::ReadFile(gzFile file)
 						// Expand memory block
 		newBuffer = static_cast<char *>(realloc(buffer, bufferSize));
 		if (newBuffer == NULL)
-			throw BAD_ALLOC();
+			throw bad_alloc();
 			
 		buffer = newBuffer;
 	}
