@@ -1,6 +1,6 @@
 /*
 	khwin.cc	Hypertext Window
-	Copyright (c) 1997-8,2000,2002,2003 Kriang Lerdsuwanakij
+	Copyright (c) 1997-8,2000,2002,2003,2004,2007,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -251,8 +251,8 @@ int	khScreenManager::ReadKeyboard()
 			if (c == KEY_F(8))		// Refresh screen
 				RequestRefresh();
 							// ^R or ^L
-			else if (c == 'R'-'A'+1 && kcdConfig.cfgKey == keyBindingDefault
-				 || c == 'L'-'A'+1 && kcdConfig.cfgKey == keyBindingVi)
+			else if ((c == 'R'-'A'+1 && kcdConfig.cfgKey == keyBindingDefault)
+				 || (c == 'L'-'A'+1 && kcdConfig.cfgKey == keyBindingVi))
 				RequestRefresh();
 							// ^F
 			else if (c == 'F'-'A'+1 && kcdConfig.cfgKey == keyBindingDefault) {
@@ -437,7 +437,7 @@ void	HyperWindow::Init()
 					// Our new pad
 	win = newpad(GetDocRow(), numRealColumn);
 	if (win == NULL)
-		throw BAD_ALLOC();
+		throw bad_alloc();
 
 	if (win->_line == NULL) {	// Some macros are broken due to
 					// different bool size
@@ -456,7 +456,7 @@ void	HyperWindow::Init()
 			
 					// This shouldn't happen
 		if (blankWin == NULL)
-			throw BAD_ALLOC();
+			throw bad_alloc();
 
 		wbkgd(blankWin, normalA);		// Set normal attr as background
 		wattrset(blankWin, normalA);		// Set normal text attr
@@ -2065,9 +2065,9 @@ void	HyperWindow::ProcessKey(int ch)
 			ProcessKeyBeginLine();
 		else if (ch == 'E'-'A'+1)		// C-e
 			ProcessKeyEndLine();
-		else if (ch == 'B'-'A'+1 || ch == 'b' && emacsFoundAlt)	// C-b, M-b
+		else if (ch == 'B'-'A'+1 || (ch == 'b' && emacsFoundAlt))	// C-b, M-b
 			ProcessKeyPrev();
-		else if (ch == 'F'-'A'+1 || ch == 'f' && emacsFoundAlt)	// C-f, M-f
+		else if (ch == 'F'-'A'+1 || (ch == 'f' && emacsFoundAlt))	// C-f, M-f
 			ProcessKeyNext();
 		else if (ch == 'P'-'A'+1)		// C-p
 			ProcessKeyUp();
@@ -2159,8 +2159,18 @@ void	khHyperWindowWithFind::FindText(const string &text, bool find_backward)
 
 	iterType newAnchorIter = curAnchorIter;
 
+#ifdef	CLIB_HAVE_REGEX
+	string str = MakeString(text);
+	regex_t *reg = MakePathRegex(text);
+#endif
+
 	do {
-		if (MatchSubString(newAnchorIter, text)) {
+		if (MatchSubString(newAnchorIter, text)
+#ifdef	CLIB_HAVE_REGEX
+		    || (MatchStr(str, (*newAnchorIter)->name)
+			&& MatchPathRegex(reg, (*newAnchorIter)->name))
+#endif
+		    ) {
 			CenterRowPadIfScroll(newAnchorIter);
 			FitHighlightMiddle(newAnchorIter);
 			curAnchorIter = newAnchorIter;
@@ -2175,6 +2185,11 @@ void	khHyperWindowWithFind::FindText(const string &text, bool find_backward)
 
 	Highlight();
 	scrnMan.RequestRestCursor();
+
+#ifdef	CLIB_HAVE_REGEX
+	if (reg)
+		regfree(reg);
+#endif
 }
 
 void	khHyperWindowWithFind::FindPrev(const string &text)
@@ -2183,8 +2198,18 @@ void	khHyperWindowWithFind::FindPrev(const string &text)
 
 	prev_loop(html.GetAnchorList(), newAnchorIter);
 
+#ifdef	CLIB_HAVE_REGEX
+	string str = MakeString(text);
+	regex_t *reg = MakePathRegex(text);
+#endif
+
 	do {
-		if (MatchSubString(newAnchorIter, text)) {
+		if (MatchSubString(newAnchorIter, text)
+#ifdef	CLIB_HAVE_REGEX
+		    || (MatchStr(str, (*newAnchorIter)->name)
+			&& MatchPathRegex(reg, (*newAnchorIter)->name))
+#endif
+		    ) {
 			CenterRowPadIfScroll(newAnchorIter);
 			FitHighlightMiddle(newAnchorIter);
 			curAnchorIter = newAnchorIter;
@@ -2196,6 +2221,11 @@ void	khHyperWindowWithFind::FindPrev(const string &text)
 
 	Highlight();
 	scrnMan.RequestRestCursor();
+
+#ifdef	CLIB_HAVE_REGEX
+	if (reg)
+		regfree(reg);
+#endif
 }
 
 void	khHyperWindowWithFind::FindNext(const string &text)
@@ -2204,8 +2234,18 @@ void	khHyperWindowWithFind::FindNext(const string &text)
 
 	next_loop(html.GetAnchorList(), newAnchorIter);
 
+#ifdef	CLIB_HAVE_REGEX
+	string str = MakeString(text);
+	regex_t *reg = MakePathRegex(text);
+#endif
+
 	do {
-		if (MatchSubString(newAnchorIter, text)) {
+		if (MatchSubString(newAnchorIter, text)
+#ifdef	CLIB_HAVE_REGEX
+		    || (MatchStr(str, (*newAnchorIter)->name)
+			&& MatchPathRegex(reg, (*newAnchorIter)->name))
+#endif
+		    ) {
 			CenterRowPadIfScroll(newAnchorIter);
 			FitHighlightMiddle(newAnchorIter);
 			curAnchorIter = newAnchorIter;
@@ -2217,6 +2257,11 @@ void	khHyperWindowWithFind::FindNext(const string &text)
 
 	Highlight();
 	scrnMan.RequestRestCursor();
+
+#ifdef	CLIB_HAVE_REGEX
+	if (reg)
+		regfree(reg);
+#endif
 }
 
 void	khHyperWindowWithFind::SelectText(const string & /*text*/)
@@ -2292,6 +2337,32 @@ void	khHyperWindowWithFind::ProcessKey(int ch)
 			scrnMan.SetCursor(&status);
 			status.ProcessKey(ch);
 
+			emacsFoundAlt = false;
+		}
+#ifdef USE_UTF8_MODE
+		else if (IsUTF8Mode()) {
+			if ((ch <= 0x7F && isalnum(ch)) || (ch > 0x7F && ch <= 0xFF))
+				start_find = true;
+		}
+		else {
+			if (ch <= 0xFF && isalnum(ch))
+				start_find = true;
+		}
+#else
+		else if (ch <= 0xFF && isalnum(ch))
+			start_find = true;
+#endif
+
+		if (start_find) {
+				// Pass control of keyboard & cursor to
+				// status
+			scrnMan.SetFocus(&status);
+			scrnMan.SetCursor(&status);
+			status.ProcessKey('S'-'A'+1);	// Assume C-s
+			status.ProcessKey(ch);
+
+			emacsFoundAlt = false;
+
 			emacsFoundAlt = false;
 		}
 		else
