@@ -1,6 +1,6 @@
 /*
 	scandir.cc	Scan Directory Functions
-	Copyright (c) 2000,2001,2003 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2003,2005,2007 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -54,7 +54,22 @@ void	CommandLineScanLog::PrintTime(long secdiff, const string &mode)
 		     << mode;
 }
 
-void	CommandLineScanLog::PrintStartDirNotFound(const string &str)
+void	CommandLineScanLog::PrintChdirError(const string &dir, const string &str)
+{
+	cout << flush;		// Avoid out-of-order display
+	gtout(cerr, _(": cannot change to directory %$ - %$\n"))
+	     << dir << str;
+}
+
+void	ScanCommandScanLog::PrintStartDirNotDirectory(const string &str)
+{
+	cout << flush;		// Avoid out-of-order display
+	gtout(cerr, _("%$: %$ specified in `StartDir\' command is not"
+		      " a directory.  Command ignored\n"))
+	     << progName << str;
+}
+
+void	ScanCommandScanLog::PrintStartDirNotFound(const string &str)
 {
 	cout << flush;		// Avoid out-of-order display
 	gtout(cerr, _("%$: cannot find directory %$"
@@ -62,11 +77,34 @@ void	CommandLineScanLog::PrintStartDirNotFound(const string &str)
 	     << progName << str;
 }
 
-void	CommandLineScanLog::PrintChdirError(const string &dir, const string &str)
+void	TreeCommandScanLog::PrintStartDirNotDirectory(const string &str)
 {
 	cout << flush;		// Avoid out-of-order display
-	gtout(cerr, _(": cannot change to directory %$ - %$\n"))
-	     << dir << str;
+	gtout(cerr, _("%$: %$ is not a directory.  Command ignored\n"))
+	     << progName << str;
+}
+
+void	TreeCommandScanLog::PrintStartDirNotFound(const string &str)
+{
+	cout << flush;		// Avoid out-of-order display
+	gtout(cerr, _("%$: cannot find directory %$.  Command ignored\n"))
+	     << progName << str;
+}
+
+void	MountCommandScanLog::PrintStartDirNotDirectory(const string &str)
+{
+	cout << flush;		// Avoid out-of-order display
+	gtout(cerr, _("%$: %$ specified in `MountDir\' command is not"
+		      " a directory.  Command ignored\n"))
+	     << progName << str;
+}
+
+void	MountCommandScanLog::PrintStartDirNotFound(const string &str)
+{
+	cout << flush;		// Avoid out-of-order display
+	gtout(cerr, _("%$: cannot find directory %$"
+		      " as specified in `MountDir\' command.  Command ignored\n"))
+	     << progName << str;
 }
 
 
@@ -96,7 +134,8 @@ void	ExpandSymLink(ScanLog &scan_log)
 void	RecursiveFullScanDir(ScanLog &scan_log, sptr<DirectoryEntry> &d, 
 			     const string &current_dir)
 {
-	if (FindSkipDir(current_dir)) {		// Match skipped dir. 
+	if (FindSkipDir(current_dir)
+	    || FindMountDir(current_dir)) {	// Match skipped dir. 
 		d->isSkip = 1;
 		return;
 	}
@@ -169,9 +208,10 @@ void	RecursiveFullScanDir(ScanLog &scan_log, sptr<DirectoryEntry> &d,
 
 void	FullScanFromDir(ScanLog &scan_log, const string &dir, bool set_flags = false)
 {
-	if (k_chdir(dir)) {
+	if (k_access (dir, F_OK))
 		scan_log.PrintStartDirNotFound(dir);
-	}
+	else if (k_chdir(dir))
+		scan_log.PrintStartDirNotDirectory(dir);
 	else {
 		sptr<DirectoryEntry> d(new DirectoryEntry(dir));
 		dirTree.push_back(d);
@@ -210,7 +250,7 @@ void	FullScanDir(ScanLog &scan_log)
 			try {
 				FullScanFromDir(scan_log, UnquoteShellChars((*iter)->dir));
 			}
-			catch (ErrorRange &) {		// Ignore
+			catch (ErrorRange &) {		// Ignore quoting error
 			}
 		}
 		
@@ -253,9 +293,7 @@ struct	AssertDirFlag {
 	AssertDirFlag(char flags_) : flags(flags_) {}
 	void operator()(sptr<DirectoryEntry> &d) {
 		if (d->flags != flags) {
-			cout << flush;
-			cerr << progName << _(": Invalid directory tree state\n");
-			dump_core();
+			throw ErrorGeneric("Invalid directory tree state, program aborted");
 		}
 	}
 };
@@ -266,7 +304,8 @@ void	RecursiveSmartScanDir(ScanLog &scan_log, sptr<DirectoryEntry> &d,
 	int	ret;
 	int	fullDirRead = 0;
 
-	if (FindSkipDir(current_dir)) {		// Match skipped dir. 
+	if (FindSkipDir(current_dir)
+	    || FindMountDir(current_dir)) {	// Match skipped dir. 
 		d->isSkip = 1;
 		d->subDir.clear();
 		return;
@@ -525,7 +564,7 @@ void	SmartScanDir(ScanLog &scan_log)
 					FullScanFromDir(scan_log, dir_name, true);
 				}
 			}
-			catch (ErrorRange &) {		// Ignore
+			catch (ErrorRange &) {		// Ignore quoting error
 			}
 		}
 	}
@@ -542,6 +581,7 @@ void	SmartScanDir(ScanLog &scan_log)
 		if ((*iter)->IsDir()) {
 			
 			if ((*iter)->flags == 1)	// Already scanned
+							// by FullScanFromDir
 				continue;
 
 			string	new_dir = (*iter)->GetNameStr();
@@ -675,13 +715,13 @@ void	PartialScanDir(ScanLog &scan_log, const vector<string> &str_, bool full_sca
 
 void	SmartScanDir()
 {
-	CommandLineScanLog	scan_log;
+	ScanCommandScanLog	scan_log;
 	SmartScanDir(scan_log);
 }
 
 void	FullScanDir()
 {
-	CommandLineScanLog	scan_log;
+	ScanCommandScanLog	scan_log;
 	FullScanDir(scan_log);
 }
 
@@ -695,13 +735,13 @@ void	ScanDir(bool full_scan)
 
 void	PartialScanDir(const vector<string> &str_, bool full_scan)
 {
-	CommandLineScanLog	scan_log;
+	ScanCommandScanLog	scan_log;
 	PartialScanDir(scan_log, str_, full_scan);
 }
 
 void	ScanDirAndDisplay(const string &str_, bool full_scan)
 {
-	CommandLineScanLog	scan_log;
+	TreeCommandScanLog	scan_log;
 	scan_log.SetQuiet(kcdConfig.cfgQuietPartial);
 
 	FindDirInfo *find = FindDir(&dirTree, str_, false, false);
@@ -722,3 +762,16 @@ void	ScanDirAndDisplay(const string &str_, bool full_scan)
 	}
 }
 
+void	MountDirScanDir(const string &str_)
+{
+	MountCommandScanLog	scan_log;
+	scan_log.SetQuiet(true);
+
+	gtout(cout, _("scanning MountDir: %$\n")) << str_;
+
+	FindDirInfo *find = FindDir(&dirTree, str_, false, false);
+	if (find) {
+		DoPartialScanDir(scan_log, str_, true);
+	}
+	delete find;
+}
