@@ -1,6 +1,6 @@
 /*
 	scandir.h	Scan Directory Functions Header
-	Copyright (c) 2000,2001,2003,2007 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2003,2005,2007 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -37,6 +37,7 @@ class ScanLog {
 		virtual void PrintDirectory(const string &str) = 0;
 		virtual void PrintWork(const string &str) = 0;
 		virtual void PrintTime(long secdiff, const string &mode) = 0;
+		virtual void PrintStartDirNotDirectory(const string &str) = 0;
 		virtual void PrintStartDirNotFound(const string &str) = 0;
 		virtual void PrintChdirError(const string &dir, const string &str) = 0;
 		void SetQuiet(bool quiet_) { quiet = quiet_; }
@@ -48,15 +49,33 @@ class CommandLineScanLog : public ScanLog {
 		void PrintDirectory(const string &str) { if (!quiet) cout << str << '\n'; }
 		void PrintWork(const string &str) { if (!quiet) cout << str << '\n'; }
 		void PrintTime(long secdiff, const string &mode);
-		void PrintStartDirNotFound(const string &str);
 		void PrintChdirError(const string &dir, const string &str);
 		virtual ~CommandLineScanLog() {}
 };
 
+class ScanCommandScanLog : public CommandLineScanLog {
+	public:
+		void PrintStartDirNotDirectory(const string &str);
+		void PrintStartDirNotFound(const string &str);
+};
+
+class TreeCommandScanLog : public CommandLineScanLog {
+	public:
+		void PrintStartDirNotDirectory(const string &str);
+		void PrintStartDirNotFound(const string &str);
+};
+
+class MountCommandScanLog : public CommandLineScanLog {
+	public:
+		void PrintStartDirNotDirectory(const string &str);
+		void PrintStartDirNotFound(const string &str);
+};
+
 void	FullScanDir();
 void	SmartScanDir();
 void	ScanDir(bool full_scan);
 void	PartialScanDir(const vector<string> &str_, bool full_scan);
 void	ScanDirAndDisplay(const string &str_, bool full_scan);
+void	MountDirScanDir(const string &str_);
 
 #endif	/* __K_SCANDIR_H */
