@@ -21,7 +21,15 @@
 #ifndef __K_SETUPCURSES_H
 #define __K_SETUPCURSES_H
 
-#include <curses.h>
+#ifdef NCURSESAC_NEW_INCLUDE
+# ifdef NCURSESAC_WIDE
+#  include <ncursesw/curses.h>
+# else
+#  include <ncurses/curses.h>
+# endif
+#else
+# include <curses.h>
+#endif
 
 //
 // Remove conflict macros
