@@ -1,6 +1,6 @@
 /*
 	statwin.cc	Status Window
-	Copyright (c) 1997-8,2000,2003 Kriang Lerdsuwanakij
+	Copyright (c) 1997-8,2000,2003,2004,2007,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -80,7 +80,7 @@ void	khStatusWindow::Init()
 	if (GetRow() && GetCol()) {
 		win = newwin(GetRow(), GetCol(), GetY(), GetX());
 		if (win == NULL) {		// Error
-			throw BAD_ALLOC();
+			throw bad_alloc();
 		}
 	}
 	else
@@ -164,7 +164,7 @@ void	khURLWindow::Init()
 	if (GetRow() && GetCol()) {
 		win = newwin(GetRow(), GetCol(), GetY(), GetX());
 		if (win == NULL) {		// Error
-			throw BAD_ALLOC();
+			throw bad_alloc();
 		}
 	}
 	else
@@ -209,9 +209,32 @@ void	khURLWindow::ProcessMouse(MEVENT &event)
 {
 			// Only accepts mouse button 1
 	if (!(event.bstate & (BUTTON1_CLICKED | BUTTON1_DOUBLE_CLICKED
-			      | BUTTON1_TRIPLE_CLICKED | BUTTON1_RELEASED)))
+			      | BUTTON1_TRIPLE_CLICKED | BUTTON1_PRESSED)))
 		return;
 
+	if (event.bstate & BUTTON1_TRIPLE_CLICKED) {
+		// Convert triple click to 3 separate clicks.
+		event.bstate ^= BUTTON1_TRIPLE_CLICKED;
+		event.bstate |= BUTTON1_CLICKED;
+		ProcessMouse(event);
+		scrnMan.RequestUpdate();
+		ProcessMouse(event);
+		scrnMan.RequestUpdate();
+		ProcessMouse(event);
+		scrnMan.RequestUpdate();
+		return;
+	}
+	if (event.bstate & BUTTON1_DOUBLE_CLICKED) {
+		// Convert double click to 2 separate clicks.
+		event.bstate ^= BUTTON1_DOUBLE_CLICKED;
+		event.bstate |= BUTTON1_CLICKED;
+		ProcessMouse(event);
+		scrnMan.RequestUpdate();
+		ProcessMouse(event);
+		scrnMan.RequestUpdate();
+		return;
+	}
+
 	NextStatus();
 }
 #endif
@@ -356,12 +379,13 @@ khURLWindowWithFind::khURLWindowWithFind(NCScreenManager &scrnMan_, int id_) :
 			break;
 		case keyBindingVi:
 			statKey4WithFind = _("Keys (4v):  /,?=Enter find mode");
-			statKeyFind = _("Keys in find mode:  TAB=Exit find  ENTER=Select");
+			statKeyFind = _("Keys in find mode:  Up,?=Prev  Dn,/=Next"
+					"  TAB=Exit find  ENTER=Select");
 			statKeyCtrlF = _("^F:  8=Repaint screen  9=Switch display"
 					 "  0=Quit");
 			break;
 		case keyBindingEmacs:
-			statKey4WithFind = _("Keys (4e):  C-s,C-r=Enter find mode");
+			statKey4WithFind = _("Keys (4e):  C-s,C-r,A-Z,a-z,0-9=Enter find mode");
 			statKeyFind = _("Keys in find mode:  Up/Dn=Prev/next match"
 					"  TAB=Exit find  ENTER=Select");
 			statKeyCtrlF = _("C-f:  8=Repaint screen  9=Switch display"
@@ -378,10 +402,10 @@ khURLWindowWithFind::khURLWindowWithFind(NCScreenManager &scrnMan_, int id_) :
 
 const int	khStatusWindowWithFind::MAX_SEARCHCHAR;
 
-char	*khStatusWindowWithFind::statFind1 = N_("Find: [");
-char	*khStatusWindowWithFind::statFind2 = N_("]  ");
-char	*khStatusWindowWithFind::statFindOvr = N_("Ovr");
-char	*khStatusWindowWithFind::statFindIns = N_("Ins");
+const char	*khStatusWindowWithFind::statFind1 = N_("Find: [");
+const char	*khStatusWindowWithFind::statFind2 = N_("]  ");
+const char	*khStatusWindowWithFind::statFindOvr = N_("Ovr");
+const char	*khStatusWindowWithFind::statFindIns = N_("Ins");
 
 int	khStatusWindowWithFind::spaceSelect = 1;	// Default = true
 
@@ -549,14 +573,14 @@ void	khStatusWindowWithFind::ProcessKey(int ch)
 			findText->FindProcessKey(ch);
 		}
 		else if (ch == KEY_DOWN || ch == KEY_NPAGE
-			 || kcdConfig.cfgKey == keyBindingVi && ch == '/'
-			 || kcdConfig.cfgKey == keyBindingEmacs && ch == 'S'-'A'+1) {
+			 || (kcdConfig.cfgKey == keyBindingVi && ch == '/')
+			 || (kcdConfig.cfgKey == keyBindingEmacs && ch == 'S'-'A'+1)) {
 			findText->FindNext(edit_string);
 			return;
 		}
 		else if (ch == KEY_UP || ch == KEY_PPAGE
-			 || kcdConfig.cfgKey == keyBindingVi && ch == '?'
-			 || kcdConfig.cfgKey == keyBindingEmacs && ch == 'R'-'A'+1) {
+			 || (kcdConfig.cfgKey == keyBindingVi && ch == '?')
+			 || (kcdConfig.cfgKey == keyBindingEmacs && ch == 'R'-'A'+1)) {
 			findText->FindPrev(edit_string);
 			return;
 		}
@@ -635,7 +659,7 @@ void	khMarkerBar::Init()
 	if (GetRow() && GetCol()) {
 		win = newwin(GetRow(), GetCol(), GetY(), GetX());
 		if (win == NULL) {		// Error
-			throw BAD_ALLOC();
+			throw bad_alloc();
 		}
 	}
 	else
