@@ -1,6 +1,6 @@
 /*
 	strmisc.cc	String Functions
-	Copyright (c) 1997,1998,1999,2000,2001,2003 Kriang Lerdsuwanakij
+	Copyright (c) 1997-9,2000-1,2003,2004,2007,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,15 @@
 
 #ifdef USE_UTF8_MODE
 # ifdef HAVE_WCHAR_H
-#  define __USE_XOPEN
+#  ifndef __USE_XOPEN
+#   define __USE_XOPEN
+#  endif
 #  include CXX__HEADER_cwchar
 # endif
 #endif
 
+#include CXX__HEADER_cstdlib
+
 /*************************************************************************
 	String to number conversion
 *************************************************************************/
@@ -99,6 +103,7 @@ int	CompareStringCase(const char *str1, const char *str2, size_t n)
 *************************************************************************/
 
 char	charNeedGlobQuote[] = "*?[]\\";
+char	charNeedRegexQuote[] = "^.[]$()|*+?{}\\";
 char	charNeedShellQuote[] = "\'\"`\\";
 char	charNoNeedURLQuote[] = "/$-_@.&+-";	// Slash + safe chars
 char	charHex[] = "0123456789abcdef";
@@ -174,6 +179,39 @@ string	UnquoteShellChars(const string &src, bool quoteGlob)	// Remove quotes
 	return dest;
 }
 
+string	UnquoteConfigChars(const string &src)		// Remove quotes
+{
+	string::size_type	len = src.size();
+	string	dest;
+	bool	quoted = false;
+	for (string::size_type i = 0; i < len; i++) {
+		if (quoted) {
+			switch (src[i]) {
+				case 'x':
+					if (len < i+3)
+						throw ErrorRange();
+					dest += HexToBin(static_cast<unsigned char>(src[i+1]))*16
+							   +HexToBin(static_cast<unsigned char>(src[i+2]));
+					i += 2;
+					break;
+				case '\"':
+					dest += src[i];
+				default:
+					dest += '\\';
+					dest += src[i];
+			}
+			quoted = false;
+		}
+		else if (src[i] == '\\') {
+			quoted = true;
+		}
+		else {
+			dest += src[i];
+		}
+	}
+	return dest;
+}
+
 string 	QuoteURLChars(const string &src)	// Quotes special symbols
 {
 	string::size_type	len = src.size();
@@ -225,6 +263,22 @@ string 	QuoteGlobChars(const string &src)	// Quotes special symbols
 	return dest;
 }
 
+string 	QuoteRegexChars(const string &src)	// Quotes special symbols
+{
+	string::size_type	len = src.size();
+	string	dest;
+	for (string::size_type i = 0; i < len; i++) {
+		if (strchr(charNeedRegexQuote, src[i])) {
+			dest += '\\';
+			dest += src[i];
+		}
+		else {
+			dest += src[i];
+		}
+	}
+	return dest;
+}
+
 /*************************************************************************
 	Check last part of string
 *************************************************************************/
@@ -250,7 +304,7 @@ string::size_type	StrLast(const string &str, const string &last)
 	Unicode support
 *************************************************************************/
 
-void	k_wstring::operator=(char *s)
+void	k_wstring::operator=(const char *s)
 {
 	len = 0;
 	while(*s) {
@@ -327,7 +381,7 @@ size_t	GetStringWidth(const string &s)
 		memset(&state, 0, sizeof(mbstate_t));
 		size_t	num = mbsrtowcs(buf(), &p, s.size()+1, &state);
 		if (num == static_cast<size_t>(-1))
-			throw ErrorBadSequence();
+			throw ErrorGenericSyntax(_("bad multibyte sequence %$"), s);
 
 		size_t size = 0;
 		for (size_t i = 0; i < num; ++i)
@@ -396,3 +450,78 @@ size_t	GetWStringWidthLastCharAtPos(const k_wstring &s, size_t pos)
 {
 	return GetWStringWidth(s, GetWStringLastCharAtPos(s, pos));
 }
+
+/*************************************************************************
+	/, // Regex Directory Match
+*************************************************************************/
+
+bool	MatchPathRegex(regex_t *reg, const string &dir)
+{
+#ifdef	CLIB_HAVE_REGEX
+	if (!reg)
+		return true;
+
+	regmatch_t m;
+	return !regexec(reg, dir.c_str(), 1, &m, 0);
+#else
+	return true;
+#endif
+}
+
+bool	MatchStr(const string &str, const string &dir)
+{
+	size_t slash_pos = dir.rfind('/');
+	size_t link_pos;
+	if (slash_pos != string::npos)
+		link_pos = dir.find(str, slash_pos+1);
+	else
+		link_pos = dir.find(str);
+
+	return link_pos != string::npos;
+}
+
+regex_t	*MakePathRegex(const string &search_str)
+{
+#ifdef	CLIB_HAVE_REGEX
+	size_t orig_pos = 0;
+	size_t slash_pos = search_str.find('/');
+	if (slash_pos == string::npos)
+		return 0;
+
+	string regex = ".*";
+
+	do {
+		regex.append(QuoteRegexChars(search_str.substr(orig_pos, slash_pos-orig_pos)));
+		if (search_str[slash_pos+1] == '/') {
+			regex.append(".*/.*");
+			slash_pos++;
+		}
+		else
+			regex.append("[^/]*/[^/]*");
+
+		orig_pos = slash_pos+1;
+		slash_pos = search_str.find('/', orig_pos);
+	} while (slash_pos != string::npos);
+
+	regex.append(1, '$');
+	regex_t *reg = new regex_t;
+	if (regcomp(reg, regex.c_str(), REG_EXTENDED))
+		return 0;		// Error
+	return reg;
+#else
+	return 0;
+#endif
+}
+
+string	MakeString(const string &search_str)
+{
+#ifdef	CLIB_HAVE_REGEX
+	size_t slash_pos = search_str.rfind('/');
+	if (slash_pos == string::npos)
+		return search_str;
+	else
+		return string(search_str, slash_pos+1);
+#else
+	return search_str;
+#endif
+}
