@@ -1,6 +1,6 @@
 /*
 	strmisc.h	String Header
-	Copyright (c) 1997-8,2000,2003 Kriang Lerdsuwanakij
+	Copyright (c) 1997-8,2000,2003,2004,2009 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,13 @@
 
 #include "error.h"
 
+#ifdef	CLIB_HAVE_REGEX
+# include <sys/types.h>
+# include <regex.h>
+#else
+  typedef int regex_t;
+#endif
+
 USING_NAMESPACE_STD;
 
 // Throw ErrorRange if error
@@ -63,9 +70,11 @@ inline int	CompareStringCase(const string &str1, const string &str2,
 
 string 	QuoteShellChars(const string &src);	// Quotes special symbols
 string	UnquoteShellChars(const string &src, bool quoteGlob = false);	// Remove quotes
+string	UnquoteConfigChars(const string &src);	// Remove quotes
 string 	QuoteURLChars(const string &src);	// Quotes special symbols
 string	UnquoteURLChars(const string &src);	// Remove quotes
 string 	QuoteGlobChars(const string &src);	// Quotes special symbols
+string 	QuoteRegexChars(const string &src);	// Quotes special symbols
 
 string::size_type	StrLast(const string &str, const string &last);
 
@@ -81,7 +90,7 @@ class k_wstring {
 		const wchar_t&	operator[](size_t n) const { return buf[n]; }
 		const wchar_t*	data() const { return buf; }
 
-		void	operator=(char *s);
+		void	operator=(const char *s);
 		void	erase(size_t pos, size_t n);
 		void	insert(size_t pos, size_t n, wchar_t c);
 };
@@ -94,4 +103,9 @@ size_t	GetWStringWidth(const k_wstring &s, size_t n);
 size_t	GetWStringLastCharAtPos(const k_wstring &s, size_t pos);
 size_t	GetWStringWidthLastCharAtPos(const k_wstring &s, size_t pos);
 
+bool	MatchStr(const string &str, const string &dir);
+bool	MatchPathRegex(regex_t *reg, const string &dir);
+regex_t	*MakePathRegex(const string &search_str);
+string	MakeString(const string &search_str);
+
 #endif	/* __K_STRMISC_H */
