@@ -1,6 +1,6 @@
 /*
 	utf8dec.h	UTF-8 to wchar_t Decoder Header
-	Copyright (c) 2000,2001 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2007 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,9 @@
 
 #ifdef USE_UTF8_MODE
 # ifdef HAVE_WCHAR_H
-#  define __USE_XOPEN
+#  ifndef __USE_XOPEN
+#   define __USE_XOPEN
+#  endif
 #  include CXX__HEADER_cwchar
 #  include CXX__HEADER_cwctype
 # endif
